/** 
 * @file yfrobot
 * @brief YFROBOT's sensors Mind+ library.
 * @n This is a MindPlus graphics programming extension for YFROBOT's module.
 * 
 * @copyright    YFROBOT,2022
 * @copyright    MIT Lesser General Public License
 * 
 * @author [email](yfrobot@qq.com)
 * @date  2023-03-07
*/

enum ADDR {
    //% block="0x70(default)"
    0x70,
    //% block="0x71"
    0x71,
    //% block="0x72"
    0x72,
    //% block="0x73"
    0x73,
    //% block="0x74"
    0x74,
    //% block="0x75"
    0x75,
    //% block="0x76"
    0x76,
    //% block="0x77"
    0x77
}

enum I2CEXTPORT {
    //% blockId="Port0" block="1"
    0,
    //% blockId="Port1" block="2"
    1,
    //% blockId="Port2" block="3"
    2,
    //% blockId="Port3" block="4"
    3
}

//% color="#96c24e" iconWidth=50 iconHeight=40 芽绿
namespace i2cext {

    //% block="Initializes the I2C module address is [I2CMODULEADDR]" blockType="command"
    //% I2CMODULEADDR.shadow="dropdown" I2CMODULEADDR.options="ADDR" I2CMODULEADDR.defl="ADDR.0x70"
    export function init(parameter: any, block: any) {
        let i2c_addr = parameter.I2CMODULEADDR.code;
        Generator.addInclude(`include_i2c_multiplexer`, `#include <DFRobot_I2CMultiplexer.h>`);
        Generator.addObject(`object_i2c_multiplexer`, `DFRobot_I2CMultiplexer`, `I2CMulti;`);
        Generator.addSetup(`I2CMulti.begin_${i2c_addr}`,`I2CMulti.begin(${i2c_addr});`);
    }

    //% block="Select Port [PORT]" blockType="command"
    //% PORT.shadow="dropdown" PORT.options="I2CEXTPORT"  PORT.defl="I2CEXTPORT.0" 
    export function selectPort(parameter: any, block: any) {
        let port = parameter.PORT.code;
        Generator.addCode(`I2CMulti.selectPort(${port});`);

    }
}
