/** 
 * @file yfrobot
 * @brief YFROBOT's 4wd mcnam whell smart car Mind+ library.
 * @n This is a MindPlus graphics programming extension for YFROBOT's module.
 * 
 * @copyright    YFROBOT,2022
 * @copyright    MIT Lesser General Public License
 * 
 * @author [email](yfrobot@qq.com)
 * @date  2022-06-13
*/
enum HIGHLOW {
    //% block="LOW"
    LOW,
    //% block="HIGH"
    HIGH
}

enum MWDIR {
    //% block="FORWARD"
    0,
    //% block="BACK OFF"
    1,
    //% block="RIGHT"
    2,
    //% block="LEFT"
    3
}

enum MOTORNUM {
    //% block="MA"
    MA,
    //% block="MB"
    MB,
}

//% color="#bdaead" iconWidth=50 iconHeight=40
namespace legoduinoMotor {

    //% block="legoduino motor MA [SPEEDMA] MB [SPEEDMB] " blockType="command"
    //% SPEEDMA.shadow="range"   SPEEDMA.params.min=-255    SPEEDMA.params.max=255    SPEEDMA.defl=200
    //% SPEEDMB.shadow="range"   SPEEDMB.params.min=-255    SPEEDMB.params.max=255    SPEEDMB.defl=200
    export function legoduinoMotorDrive(parameter: any, block: any) {
        let speedMA = parameter.SPEEDMA.code;
        let speedMB = parameter.SPEEDMB.code;

        Generator.addInclude(`definelegoduinomotor`, `PROGMEM void legoMotorDrive(int mASpeed, int mBSpeed); // 麦轮电机控制函数`)
        Generator.addInclude(`definelegoduinomotorFun`, `/*\n` +
            `  LEGODUINO 驱动电机函数\n` +
            `  参数:mASpeed - A电机速度 mBSpeed - B电机速度(取值范围：-255 ~ 255)\n` +
            `*/\n` +
            `void legoMotorDrive(int mASpeed, int mBSpeed) {\n` +
            `  if (mASpeed > 0) {\n` +
            `    digitalWrite(4, LOW);\n` +
            `  } else if (mASpeed < 0) {\n` +
            `    digitalWrite(4, HIGH);\n` +
            `  } \n` +
            `  analogWrite(5, abs(mASpeed));\n` +
            `  if (mBSpeed > 0) {\n` +
            `    digitalWrite(7, LOW);\n` +
            `  } else if (mBSpeed < 0) {\n` +
            `    digitalWrite(7, HIGH);\n` +
            `  }\n` +
            `  analogWrite(6, abs(mBSpeed));\n` +
            `}`
        );

        Generator.addCode(`legoMotorDrive(${speedMA},${speedMB});`);
    }

    
    //% block="legoduino motor [MOTOR] SPEED [SPEED] " blockType="command"
    //% MOTOR.shadow="dropdown" MOTOR.options="MOTORNUM" MOTOR.defl="MOTORNUM.MA"
    //% SPEED.shadow="range"   SPEED.params.min=-255    SPEED.params.max=255    SPEED.defl=200
    export function legoduinoMotorDrive_Single(parameter: any, block: any) {
        let motor = parameter.MOTOR.code;
        let speed = parameter.SPEED.code;

        Generator.addInclude(`definelegoduinomotor`, `PROGMEM void legoMotorDrive(int mASpeed, int mBSpeed); // 麦轮电机控制函数`)
        Generator.addInclude(`definelegoduinomotorFun`, `/*\n` +
            `  LEGODUINO 驱动电机函数\n` +
            `  参数:mASpeed - A电机速度 mBSpeed - B电机速度(取值范围：-255 ~ 255)\n` +
            `*/\n` +
            `void legoMotorDrive(int mASpeed, int mBSpeed) {\n` +
            `  if (mASpeed > 0) {\n` +
            `    digitalWrite(4, LOW);\n` +
            `  } else if (mASpeed < 0) {\n` +
            `    digitalWrite(4, HIGH);\n` +
            `  } \n` +
            `  analogWrite(5, abs(mASpeed));\n` +
            `  if (mBSpeed > 0) {\n` +
            `    digitalWrite(7, LOW);\n` +
            `  } else if (mBSpeed < 0) {\n` +
            `    digitalWrite(7, HIGH);\n` +
            `  }\n` +
            `  analogWrite(6, abs(mBSpeed));\n` +
            `}`
        );
        if(motor == "MA"){
            Generator.addCode(`legoMotorDrive(${speed},0);`);
        } else {
            Generator.addCode(`legoMotorDrive(0,${speed});`);
        }
    }

    //% block="legoduino motor [DIR] at [SPE] speed" blockType="command"
    //% DIR.shadow="dropdown" DIR.options="MWDIR" DIR.defl="MWDIR.FORWARD"
    //% SPE.shadow="range"   SPE.params.min=0    SPE.params.max=255    SPE.defl=200
    export function legoduinoMotorMove(parameter: any, block: any) {
        let dir = parameter.DIR.code;
        let spe = parameter.SPE.code;

        Generator.addInclude(`definelegoduinomotor`, `PROGMEM void legoMotorDrive(int mASpeed, int mBSpeed); // 麦轮电机控制函数`)
        Generator.addInclude(`definelegoduinomotorFun`, `/*\n` +
            `  LEGODUINO 驱动电机函数\n` +
            `  参数:mASpeed - A电机速度 mBSpeed - B电机速度(取值范围：-255 ~ 255)\n` +
            `*/\n` +
            `void legoMotorDrive(int mASpeed, int mBSpeed) {\n` +
            `  if (mASpeed > 0) {\n` +
            `    digitalWrite(4, LOW);\n` +
            `  } else if (mASpeed < 0) {\n` +
            `    digitalWrite(4, HIGH);\n` +
            `  } \n` +
            `  analogWrite(5, abs(mASpeed));\n` +
            `  if (mBSpeed > 0) {\n` +
            `    digitalWrite(7, LOW);\n` +
            `  } else if (mBSpeed < 0) {\n` +
            `    digitalWrite(7, HIGH);\n` +
            `  }\n` +
            `  analogWrite(6, abs(mBSpeed));\n` +
            `}`
        );

        if (dir == `0`) {
            Generator.addCode(`legoMotorDrive(${spe},${spe});`);
        } else if (dir == `1`) {
            Generator.addCode(`legoMotorDrive(0-${spe},0-${spe});`);
        } else if (dir == `2`) {
            Generator.addCode(`legoMotorDrive(${spe},${spe}/2);`);
        } else if (dir == `3`) {
            Generator.addCode(`legoMotorDrive(${spe}/2,${spe});`);
        }
    }

    //% block="legoduino motor STOP" blockType="command"
    export function legoduinoMotorStop(parameter: any, block: any) {
        Generator.addInclude(`definelegoduinomotor`, `PROGMEM void legoMotorDrive(int mASpeed, int mBSpeed); // 麦轮电机控制函数`)
        Generator.addInclude(`definelegoduinomotorFun`, `/*\n` +
            `  LEGODUINO 驱动电机函数\n` +
            `  参数:mASpeed - A电机速度 mBSpeed - B电机速度(取值范围：-255 ~ 255)\n` +
            `*/\n` +
            `void legoMotorDrive(int mASpeed, int mBSpeed) {\n` +
            `  if (mASpeed > 0) {\n` +
            `    digitalWrite(4, LOW);\n` +
            `  } else if (mASpeed < 0) {\n` +
            `    digitalWrite(4, HIGH);\n` +
            `  } \n` +
            `  analogWrite(5, abs(mASpeed));\n` +
            `  if (mBSpeed > 0) {\n` +
            `    digitalWrite(7, LOW);\n` +
            `  } else if (mBSpeed < 0) {\n` +
            `    digitalWrite(7, HIGH);\n` +
            `  }\n` +
            `  analogWrite(6, abs(mBSpeed));\n` +
            `}`
        );

        Generator.addCode(`legoMotorDrive(0,0);`);
    }
}
